#ifndef DFT_h
#define DFT_h

/***********************************************************
 Description: Data structures used on DFT algorithm 
 Author: Carol Hernandez
 Date of Creation Date: October 9, 1997
 Followed by Ying : December 21, 1997 
*************************************************************/
 
#define LONG long
#define BYTE  char

#define BUFFER_SIZE 5
#define FRAME_COUNT 4
#define EDI_COUNT   3
#define NRULES      5

#define DFT_THRESHOLD 604800  /* 168 hours * 60 = 10080 minutes = 604800 sec */
#define TUPLE_THRESHOLD 168  /* 2.8 minutes * 60 = 168 sec */


/* Day and hour defined in seconds */
#define ONE_DAY     86400   /* 1440 minutes */
#define ONE_HOUR    3600    /* 60 minutes */
#define INTERVAL    300     /* 5 minutes */


/* 
   The event (error) categories defined in the binary error log are:
   CPU = 0
   MEMORY = 1
   DISK = 2
   TAPE = 3
   CONTROL (Device Controllers)=4
   ADAPTER (Adapters) = 5
   BUS = 6
   INTRT (Stray Interrupts) = 7
   CONSOLE (Console Events) = 8, STACK (Stack Dumps) = 9, 
   RAID (Raid Controller) = 10, SCSI (SCSI CAM Event) = 11,
   SW_CI (CI port-to-port driver) = 12, COMM (System Communications) = 13
   
   These events are system dependent and are correlated with the events
   extracted by the error log parser
*/

#define NDEVICES 14

enum FRU_Type { CPU, MEMORY, DISK, TAPE, CONTROL, ADAPTER,
		BUS, INTRT, CONSOLE, STACK, RAID, SCSI, SW_CI, COMM };

static enum FRU_Type fruType[NDEVICES] = { 
     CPU,  MEMORY, DISK, TAPE, CONTROL, ADAPTER,
     BUS, INTRT, CONSOLE, STACK, RAID, SCSI, SW_CI, COMM 
     };

static char *fruName[NDEVICES] = { 
     "cpu", "memory", "disk", "tape", "controller", 
     "adapter", "bus","interrupt", "console", "stack", 
     "raid", "scsi", "sw_ci", "communication"
     };

struct Element {
  LONG  arrivalTime;
  int   frameSize;  /* interarrival time */
};
 
struct FRU {
  FRU_Type type;
  char     *name;
  struct   Element buffer[BUFFER_SIZE];  /* circular buffer */
  int      top;  /* index to last element in buffer */
  int      counter; /* number of filled entries in the buffer */
  int      edi[FRAME_COUNT][FRAME_COUNT-1];  /* EDIs per frame */
  LONG     lastArrivalTime;  /* Time of arrival of most recent error */
  int      rulesFired[NRULES];
  LONG     errorCount;
};

/* 
   Define an FRU (Field Replacement Unit) for every device (error category)
   monitored - System dependent
*/

/* Interface */

void  resetBuffer(struct FRU& deviceId);
void  addBufferEntry(struct FRU& deviceId, Element element);
int recordError(LONG arrivalTime, FRU& deviceId);
int applyDFTRules(FRU& deviceId);
int test4in1Rule(FRU& deviceId);
int test3_3Rule(FRU& deviceId);
int test2_2Rule(FRU& deviceId);
int test4DecreasingRule(FRU& deviceId);
void computeEDIs(FRU& deviceId);
int frame(int top, int number);
void initFRU(FRU& deviceId, enum FRU_Type type, char *name);

#endif

  
