#ifndef reliabilityAPI_h
#define reliabilityAPI_h

/**********************************************************
  Reliability API
  Author: Carol Hernandez
  Followed by Ying : Dec. 21, 1997
  
  float getAvailability(): 
     Computes the system's availability at time of request
  int   deviceStatus(int deviceId): 
     Returns the status of the device by applying the DFT rules
     to the error log for that device. Returns 1 if a warning 
     has been issued and 0 if no warning has been issued.
     A warning means that the spatial and temporal distribution
     of errors predict a possible failure of the device.
  int   lastWarning(int deviceId):
     Returns the time of the last warning or -1 if no warning has 
     issued for that device.

  The values for deviceId are as follows:
   CPU = 0
   MEMORY = 1
   DISK = 2
   TAPE = 3
   CONTROL (Device Controllers)=4
   ADAPTER (Adapters) = 5
   BUS = 6
   INTRT (Stray Interrupts) = 7
   CONSOLE (Console Events) = 8 
   STACK (Stack Dumps) = 9
   RAID (Raid Controller) = 10 
   SCSI (SCSI CAM Event) = 11
   SW_CI (CI port-to-port driver) = 12
   COMM (System Communications) = 13

   These values correspond to the event (error) categories defined 
   in the binary error log.
*********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>
#include <string.h>

#define NDEVICES 14

FILE *timeRef;
static char timeRefFile[] = "timeRef.avail";
FILE *downTime;
static char downTimeFile[] = "downTime.avail";

FILE *history;
static char historyFile[] = "crashHistory.out";
FILE *status;
static char statusFile[] = "status.out";
FILE *avail;
static char availFile[] = "availHistory.out";

static char *fruName[NDEVICES] = { 
     "cpu", "memory", "disk", "tape", "controller", 
     "adapter", "bus","interrupt", "console", "stack", 
     "raid", "scsi", "sw_ci", "communication"
     };


float getAvailability(time_t current_time); 
int   deviceStatus(int deviceId);
int   lastWarning(int deviceId);


/*********************************************************************
  Other Information provided by the reliability module which may be 
  used to implement other API functions is contained in the following
  files:
  crashHistory.out: report of machine crashes
  Format: 
  time of crash\tdown time\tavailability

  availability.out: daily availability of the machine.
  Format:
  Date\tAvailability%\n
  Date\tAvailability%\n
  ...

  status.out: snapshot of the machine's "health" as given by the offset
  of the interval between the timeStamps created by the program
  crashMonitor from the time interval specified. 
  Format:
  **** Date ****
  offset1\toffset2\toffset3\toffset4\toffset5\offset6\offset7\toffset8\n
  ...
  **** Date ****
  offset1\toffset2\toffset3\toffset4\toffset5\offset6\offset7\toffset8\n
  ...

  devicename.out : history of warnings issued for that device
  Format: 
  rule fired\ttime of warning

  The type of information that can be obtained from these files 
  includes but it is not limited to:
  Availability of the machine for a particular date
  Availability of the machine for a particular week
  Availability of the machine for a particular month
  History of Machine Crashes
  Indirect measurement of machine status (from status.out)
  History of warnings issued per device

********************************************************************/


/*********************************************************************
  List of OS services required to implement the reliability API:

  - Time functions that return current time and convert time variable
  between different formats: ascii, integer (time_t), tm structure
  time(), localtime(), mktime(), ctime()

  - Process communication functions: socket() and related functions
 
  - Timing functions: sleep()

  - Functions to access error logs (for binary logs) such as uerf

  Other OS requirements:

  - Error logs providing time of error and device causing the error

  - Authorization to access error logs

********************************************************************/

#endif
